@@ -34,13 +34,13 @@ solution.
 
  *  **Avoid computing twice the same thing**: code is rerun over an
     over, for instance when prototyping computational-heavy jobs (as in
-    scientific development), but hand-crafted solution to aleviate this
+    scientific development), but hand-crafted solution to alleviate this
     issue is error-prone and often leads to unreproducible results
 
  *  **Persist to disk transparently**: persisting in an efficient way
     arbitrary objects containing large data is hard. Using
     joblib's caching mechanism avoids hand-written persistence and
-    implicitely links the file on disk to the execution context of
+    implicitly links the file on disk to the execution context of
     the original Python object. As a result, joblib's persistence is
     good for resuming an application status or computational job, eg
     after a crash.
@@ -60,17 +60,18 @@ Main features
    inputs and  outputs: Python functions. Joblib can save their
    computation to disk and rerun it only if necessary::
 
+      >>> import numpy as np
       >>> from joblib import Memory
       >>> mem = Memory(cachedir='/tmp/joblib')
       >>> import numpy as np
-      >>> a = np.vander(np.arange(3))
+      >>> a = np.vander(np.arange(3)).astype(np.float)
       >>> square = mem.cache(np.square)
       >>> b = square(a)                                   # doctest: +ELLIPSIS
       ________________________________________________________________________________
       [Memory] Calling square...
-      square(array([[0, 0, 1],
-             [1, 1, 1],
-             [4, 2, 1]]))
+      square(array([[ 0.,  0.,  1.],
+             [ 1.,  1.,  1.],
+             [ 4.,  2.,  1.]]))
       ___________________________________________________________square - 0...s, 0.0min
 
       >>> c = square(a)
@@ -101,7 +102,7 @@ Main features
 
 """
 
-__version__ = '0.6.1'
+__version__ = '0.7.1'
 
 
 from .memory import Memory
