@@ -4,10 +4,11 @@ Helpers for logging.
 This module needs much love to become useful.
 """
 
-# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org> 
+# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
 # Copyright (c) 2008 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
+from __future__ import print_function
 
 import time
 import sys
@@ -16,6 +17,9 @@ import shutil
 import logging
 import pprint
 
+from .disk import mkdirp
+
+
 def _squeeze_time(t):
     """Remove .1s to the time under Windows: this is the time it take to
     stat files. This is needed to make results similar to timings under
@@ -26,24 +30,27 @@ def _squeeze_time(t):
     else:
         return t
 
+
 def format_time(t):
     t = _squeeze_time(t)
-    return "%.1fs, %.1fmin" % (t, t/60.)
+    return "%.1fs, %.1fmin" % (t, t / 60.)
+
 
 def short_format_time(t):
     t = _squeeze_time(t)
     if t > 60:
-        return "%4.1fmin" % (t/60.)
+        return "%4.1fmin" % (t / 60.)
     else:
         return " %5.1fs" % (t)
 
-################################################################################
+
+###############################################################################
 # class `Logger`
-################################################################################
+###############################################################################
 class Logger(object):
     """ Base class for logging messages.
     """
-    
+
     def __init__(self, depth=3):
         """
             Parameters
@@ -75,9 +82,9 @@ class Logger(object):
         return out
 
 
-################################################################################
+###############################################################################
 # class `PrintTime`
-################################################################################
+###############################################################################
 class PrintTime(object):
     """ Print and log messages while keeping track of time.
     """
@@ -92,59 +99,54 @@ class PrintTime(object):
             logfile = os.path.join(logdir, 'joblib.log')
         self.logfile = logfile
         if logfile is not None:
-            if not os.path.exists(os.path.dirname(logfile)):
-                os.makedirs(os.path.dirname(logfile))
+            mkdirp(os.path.dirname(logfile))
             if os.path.exists(logfile):
                 # Rotate the logs
                 for i in range(1, 9):
-                    if os.path.exists(logfile+'.%i' % i):
-                        try:
-                            shutil.move(logfile+'.%i' % i, 
-                                        logfile+'.%i' % (i+1))
-                        except:
-                            "No reason failing here"
+                    try:
+                        shutil.move(logfile + '.%i' % i,
+                                    logfile + '.%i' % (i + 1))
+                    except:
+                        "No reason failing here"
                 # Use a copy rather than a move, so that a process
                 # monitoring this file does not get lost.
                 try:
-                    shutil.copy(logfile, logfile+'.1')
+                    shutil.copy(logfile, logfile + '.1')
                 except:
                     "No reason failing here"
             try:
-                logfile = file(logfile, 'w')
-                logfile.write('\nLogging joblib python script\n')
-                logfile.write('\n---%s---\n' % time.ctime(self.last_time))
+                with open(logfile, 'w') as logfile:
+                    logfile.write('\nLogging joblib python script\n')
+                    logfile.write('\n---%s---\n' % time.ctime(self.last_time))
             except:
                 """ Multiprocessing writing to files can create race
                     conditions. Rather fail silently than crash the
-                    caculation.
+                    computation.
                 """
                 # XXX: We actually need a debug flag to disable this
                 # silent failure.
 
-
     def __call__(self, msg='', total=False):
         """ Print the time elapsed between the last call and the current
             call, with an optional message.
         """
         if not total:
             time_lapse = time.time() - self.last_time
-            full_msg = "%s: %s" % (msg, format_time(time_lapse) )
+            full_msg = "%s: %s" % (msg, format_time(time_lapse))
         else:
             # FIXME: Too much logic duplicated
             time_lapse = time.time() - self.start_time
-            full_msg = "%s: %.2fs, %.1f min" % (msg, time_lapse, time_lapse/60)
-        print >> sys.stderr, full_msg
+            full_msg = "%s: %.2fs, %.1f min" % (msg, time_lapse,
+                                                time_lapse / 60)
+        print(full_msg, file=sys.stderr)
         if self.logfile is not None:
             try:
                 print >> file(self.logfile, 'a'), full_msg
             except:
                 """ Multiprocessing writing to files can create race
                     conditions. Rather fail silently than crash the
-                    caculation.
+                    calculation.
                 """
                 # XXX: We actually need a debug flag to disable this
                 # silent failure.
         self.last_time = time.time()
-
-
-
