@@ -9,14 +9,17 @@ Test the logger module.
 import shutil
 import os
 import sys
-import StringIO
+import io
 from tempfile import mkdtemp
 import re
 
-import nose
-
 from ..logger import PrintTime
 
+try:
+    # Python 2/Python 3 compat
+    unicode('str')
+except NameError:
+    unicode = lambda s: s
 
 ###############################################################################
 # Test fixtures
@@ -42,19 +45,18 @@ def teardown():
 ###############################################################################
 # Tests
 def test_print_time():
-    """ A simple smoke test for PrintTime.
-    """
+    # A simple smoke test for PrintTime.
     try:
         orig_stderr = sys.stderr
-        sys.stderr = StringIO.StringIO()
+        sys.stderr = io.StringIO()
         print_time = PrintTime(logfile=os.path.join(env['dir'], 'test.log'))
-        print_time('Foo')
+        print_time(unicode('Foo'))
         # Create a second time, to smoke test log rotation.
         print_time = PrintTime(logfile=os.path.join(env['dir'], 'test.log'))
-        print_time('Foo')
+        print_time(unicode('Foo'))
         # And a third time
         print_time = PrintTime(logfile=os.path.join(env['dir'], 'test.log'))
-        print_time('Foo')
+        print_time(unicode('Foo'))
         printed_text = sys.stderr.getvalue()
         # Use regexps to be robust to time variations
         match = r"Foo: 0\..s, 0\.0min\nFoo: 0\..s, 0.0min\nFoo: " + \
