@@ -13,18 +13,24 @@
 Modified for joblib by Gael Varoquaux
 """
 
-
+import sys
 import os
 import socket
-import httplib
+try:
+    import httplib
+    import urlparse
+    from cStringIO import StringIO as BytesIO
+except ImportError:
+    # Python3k
+    import http as httplib
+    from urllib import parse as urlparse
+    from io import BytesIO
 import base64
-import urlparse
-import tempfile
-import cStringIO as StringIO
 
 from distutils import log
 from distutils.command.upload import upload
 
+
 class UploadDoc(upload):
     """Distutils command to upload Sphinx documentation."""
 
@@ -45,26 +51,26 @@ class UploadDoc(upload):
     def finalize_options(self):
         upload.finalize_options(self)
         if self.upload_file is None:
-            self.upload_file = 'doc/documentation.zip'    
+            self.upload_file = 'doc/documentation.zip'
         self.announce('Using upload file %s' % self.upload_file)
 
-
     def upload(self, filename):
-        content = open(filename,'rb').read()
+        content = open(filename, 'rb').read()
         meta = self.distribution.metadata
         data = {
             ':action': 'doc_upload',
             'name': meta.get_name(),
-            'content': (os.path.basename(filename),content),
+            'content': (os.path.basename(filename), content),
         }
         # set up the authentication
-        auth = "Basic " + base64.encodestring(self.username + ":" + self.password).strip()
+        auth = "Basic " + base64.encodestring(self.username + ":" + \
+                self.password).strip()
 
         # Build up the MIME payload for the POST data
         boundary = '--------------GHSKFJDLGDS7543FJKLFHRE75642756743254'
         sep_boundary = '\n--' + boundary
         end_boundary = sep_boundary + '--'
-        body = StringIO.StringIO()
+        body = BytesIO()
         for key, value in data.items():
             # handle multiple entries for the same name
             if type(value) != type([]):
@@ -77,7 +83,7 @@ class UploadDoc(upload):
                     fn = ""
                 value = str(value)
                 body.write(sep_boundary)
-                body.write('\nContent-Disposition: form-data; name="%s"'%key)
+                body.write('\nContent-Disposition: form-data; name="%s"' % key)
                 body.write(fn)
                 body.write("\n\n")
                 body.write(value)
@@ -87,7 +93,8 @@ class UploadDoc(upload):
         body.write("\n")
         body = body.getvalue()
 
-        self.announce("Submitting documentation to %s" % (self.repository), log.INFO)
+        self.announce("Submitting documentation to %s" % (self.repository),
+                      log.INFO)
 
         # build the Request
         # We can't use urllib2 since we need to send the Basic
@@ -100,7 +107,7 @@ class UploadDoc(upload):
         elif schema == 'https':
             http = httplib.HTTPSConnection(netloc)
         else:
-            raise AssertionError, "unsupported schema "+schema
+            raise AssertionError("unsupported schema " + schema)
 
         data = ''
         loglevel = log.INFO
@@ -108,19 +115,21 @@ class UploadDoc(upload):
             http.connect()
             http.putrequest("POST", url)
             http.putheader('Content-type',
-                           'multipart/form-data; boundary=%s'%boundary)
+                           'multipart/form-data; boundary=%s' % boundary)
             http.putheader('Content-length', str(len(body)))
             http.putheader('Authorization', auth)
             http.endheaders()
             http.send(body)
-        except socket.error, e:
+        except socket.error:
+            # Cannot use 'except as' to maintain Python 2.5 compatibility
+            e = sys.exc_info()[1]
             self.announce(str(e), log.ERROR)
             return
 
         response = http.getresponse()
         if response.status == 200:
-            self.announce('Server response (%s): %s' % (response.status, response.reason),
-                          log.INFO)
+            self.announce('Server response (%s): %s' %
+                          (response.status, response.reason), log.INFO)
         elif response.status == 301:
             location = response.getheader('Location')
             if location is None:
@@ -128,10 +137,10 @@ class UploadDoc(upload):
             self.announce('Upload successful. Visit %s' % location,
                           log.INFO)
         else:
-            self.announce('Upload failed (%s): %s' % (response.status, response.reason),
-                          log.ERROR)
+            self.announce('Upload failed (%s): %s' % \
+            (response.status, response.reason), log.ERROR)
         if self.show_response:
-            print '-'*75, response.read(), '-'*75
+            print('-' * 75 + response.read() + '-' * 75)
 
     def run(self):
         zip_file = self.upload_file
